/*
   libverisignfix.c - by Dwayne C. Litzenberger <dlitz@dlitz.net>
   2003 September 15
   Public Domain, and I'm not responsible if you hurt yourself with this.
   
   An LD_PRELOAD hack to intercept gethostbyname(3), gethostbyname_r(3) and
   gethostbyname2_r(3).  Any domain which resolves to a given IP address
   (specified at compile-time) will appear to the application to not exist.
   
   Compile this with the following command sequence:
   
   gcc -nostartfiles -shared -fPIC -Wl,-soname,libverisignfix.so.0 -o /lib/libverisignfix.so libverisignfix.c -ldl
   #echo "/lib/libverisignfix.so" >> /etc/ld.so.preload
*/

#include <dlfcn.h>
#include <netdb.h>
#include <sys/socket.h>  /* for AF_INET */
#include <netinet/in.h>
#include <stdlib.h>

#define convertaddr(a,b,c,d) ((a << 24) + (b << 16) + (c << 8) + d)
#define BADADDR htonl(convertaddr(64,94,110,11)) /* NB: comma-separated */

static struct hostent *(*real_gethostbyname)(const char *name);
static int (*real_gethostbyname2_r)(const char *name, int af,
         struct hostent *ret, char *buf, size_t buflen,
         struct hostent **result, int *h_errnop);
static int (*real_gethostbyname_r)(const char *name,
         struct hostent *ret, char *buf, size_t buflen,
         struct hostent **result, int *h_errnop);


static int call_gethostbyname_r (int which, const char *name, int af,
	struct hostent *ret, char *buf, size_t buflen,
	struct hostent **result, int *h_errnop)
{
	switch (which) {
	case 1:
		return (*real_gethostbyname_r)(name, ret,
				buf, buflen, result, h_errnop);
	case 2:
	default:
		return (*real_gethostbyname2_r)(name, af, ret,
				buf, buflen, result, h_errnop);
	}
}

static int my_gethostbyname_r (int which, const char *name, int af,
	struct hostent *ret, char *buf, size_t buflen,
	struct hostent **result, int *h_errnop)
{
	int retval;
	struct hostent my_ret;
	struct hostent *my_result;
	int my_h_errno;
	
	if (which == 2 && af != AF_INET) {
		goto passthru;
	}
	
	retval = call_gethostbyname_r(which, name, af,
			&my_ret, buf, buflen, &my_result, &my_h_errno);
	if (retval) {
		/* There was an error.  Good. */
		goto finished;
	}
	
	if (my_ret.h_addrtype == AF_INET &&
			my_ret.h_length == 4 &&
			*((unsigned long *)my_ret.h_addr) == BADADDR) {
		my_h_errno = HOST_NOT_FOUND;
		my_result = (void *)0;
		goto finished;
	}
	
finished:
	memcpy(ret, &my_ret, sizeof(struct hostent));
	if (my_result) {
		my_result = ret;
	}
	*result = my_result;
	*h_errnop = my_h_errno;
	return retval;
passthru:
	return call_gethostbyname_r(which, name, af, ret,
			buf, buflen, result, h_errnop);
}


void _init()
{
  void *libc;

  if(libc=dlopen("/lib/libc.so.6",RTLD_LAZY))
  {
    real_gethostbyname=dlsym(libc,"gethostbyname");
    real_gethostbyname_r=dlsym(libc,"gethostbyname_r");
    real_gethostbyname2_r=dlsym(libc,"gethostbyname2_r");
  }
}

struct hostent *gethostbyname (const char *name)
{
	struct hostent *retval;
	retval = (*real_gethostbyname)(name);
	if (retval->h_addrtype == AF_INET && retval->h_length == 4 &&
		*((unsigned long *)retval->h_addr) == BADADDR) {
		h_errno = HOST_NOT_FOUND;
		return (void *)0;
	}
	return retval;
}

int gethostbyname_r (const char *name,
	struct hostent *ret, char *buf, size_t buflen,
	struct hostent **result, int *h_errnop)
{
	return my_gethostbyname_r(1, name, 0, ret, 
			buf, buflen, result, h_errnop);
}

int gethostbyname2_r (const char *name, int af,
	struct hostent *ret, char *buf, size_t buflen,
	struct hostent **result, int *h_errnop)
{
	return my_gethostbyname_r(2, name, af, ret,
			buf, buflen, result, h_errnop);
}

